#include <iostream>

struct DebugStruct
{
	DebugStruct()
	{
		std::cout << "Constructor\n";
	}
	
	~DebugStruct()
	{
		std::cout << "Destructor\n";
	}
	
	void Print()
	{
		std::cout << "Salut lume\n";
	}
};

template<typename T>
class UniquePtr
{
public:
	explicit UniquePtr(T* ptr = nullptr) :
		ptr(ptr)
	{}
	
	UniquePtr(const UniquePtr& other) = delete;
	
	UniquePtr(UniquePtr&& other) :
		ptr(other.ptr)
	{
		other.ptr = nullptr;
	}
	
	~UniquePtr()
	{
		delete ptr;
	}
	
	UniquePtr& operator = (const UniquePtr& other) = delete;
	
	UniquePtr& operator = (UniquePtr&& other)
	{
		if (this == &other) return *this;
		
		delete ptr;
		ptr = other.ptr;
		other.ptr = nullptr;
	}
	
	operator bool() const
	{
		return ptr;
	}
	
	T* operator -> ()
	{
		return ptr;
	}
	
	const T* operator -> () const
	{
		return ptr;
	}
	
	T& operator * ()
	{
		return *ptr;
	}
	
	const T& operator * () const
	{
		return *ptr;
	}
	
	void Reset(T* newPtr = nullptr)
	{
		delete ptr;
		ptr = newPtr;
	}
	
	T* Release()
	{
		T* temp = ptr;
		ptr = nullptr;
		return temp;
	}

private:
	T* ptr = nullptr;
};

int main()
{
	UniquePtr<DebugStruct> a;
	a.Reset(new DebugStruct);
	a->Print();
	a.Release();
	
	//(*a).Print();
	
	return 0;
}
